#ifndef DOTEDGECONFIGURATION_H
#define DOTEDGECONFIGURATION_H

#include "Graph/IGraphEdgeConfiguration.h"

#include "DotLabelConfiguration.h"

namespace divine::gui::dot
{
    class DotEdgeConfiguration : public DotLabelConfiguration, virtual public graph::IGraphEdgeConfiguration
    {
    public:
        DotEdgeConfiguration() : DotLabelConfiguration()
        {
        }

        DotEdgeConfiguration(const DotEdgeConfiguration& other) : DotLabelConfiguration(other)
        {
            m_lineCoordinates = other.m_lineCoordinates;
            m_lineColor = other.m_lineColor;
            m_hasHeadArrow = other.m_hasHeadArrow;
            m_headArrowPoint = other.m_headArrowPoint;
            m_hasTailArrow = other.m_hasTailArrow;
            m_tailArrowPoint = other.m_tailArrowPoint;
            m_labelBoundingRect = other.m_labelBoundingRect;
        }

        const QVector<QPointF>& lineCoordinates() const override { return m_lineCoordinates; }
        void appendLineCoordinate(const QPointF& point) { m_lineCoordinates.append(point); }
        void prependLineCoordinate(const QPointF& point) { m_lineCoordinates.prepend(point); }

        const QColor& lineColor() const override { return m_lineColor; }
        void setColor(const QColor& color) { if (color.isValid()) m_lineColor = color; }

        bool hasHeadArrow() const override { return m_hasHeadArrow; }
        void setHasHeadArrow(bool hasArrow) { m_hasHeadArrow = hasArrow; }

        bool hasTailArrow() const override { return m_hasTailArrow; }
        void setHasTailArrow(bool hasArrow) { m_hasTailArrow = hasArrow; }

        const QRectF& labelBoundingRect() const override { return m_labelBoundingRect; }
        void setlabelBoundingRect(const QRectF& rect) { m_labelBoundingRect = rect; }

    private:
        QVector<QPointF> m_lineCoordinates;
        QColor m_lineColor = QColor(Qt::black);

        bool m_hasHeadArrow = true;
        QPointF m_headArrowPoint;
        bool m_hasTailArrow = false;
        QPointF m_tailArrowPoint;

        QRectF m_labelBoundingRect;
    };
}

#endif // DOTEDGECONFIGURATION_H
