#ifndef DOTPARSER_H
#define DOTPARSER_H

#include <QString>
#include <QFont>

#include "Graph/Graph.h"

#include "DotGraphConfiguration.h"
#include "DotNodeConfiguration.h"
#include "DotEdgeConfiguration.h"

namespace divine::gui::dot
{
    /*!
     * \brief Class which parses the GraphViz's dot.
     *
     * This class takes a dot formated string and a graph instance
     * and translates dot's elements into the expected interfaces.
     */
    class DotParser
    {
    public:
        DotParser(const QString& dot, graph::Graph& graph);

        void parse();

    private:
        void parseGraphConfiguration(DotGraphConfiguration& config, const QString& data);
        void parseNodeConfiguration(DotNodeConfiguration& config, const QString& name, const QString& data);
        void parseEdgeConfiguration(DotEdgeConfiguration& config, const QString& edgeDescriptor, const QString& data);
        
        /* graph parameters */
        void readGraphDimensions(DotGraphConfiguration &config, const QString& data);

        /* node parameters */
        void readNodeBoundingRect(DotNodeConfiguration& config, const QString& data);
        void readNodeFillColor(DotNodeConfiguration& config, const QString& data);
        void readNodeLabel(DotNodeConfiguration& config, const QString& nodeName, const QString& data);

        /* edge parameters */
        void readEdgeCoordinates(DotEdgeConfiguration& config, const QString& data);
        void readEdgeArrowheads(DotEdgeConfiguration& config, const QString& data);
        void readEdgeLabel(DotEdgeConfiguration& config, const QString& headNode, const QString& tailNode, const QString& data);

        /* common parameters */
        QColor readColor(const QString& data) const;
        void readLabel(DotLabelConfiguration& config, QString& labelData, const QString& data) const;
        void readLabelFont(const QString& data, QFont& font) const;
        QColor readLabelColor(const QString& data) const;
        Qt::AlignmentFlag readLabelHorizontalAlignment(QString& label) const;

        /* string manipulation */
        QString findParameter(const QString& data, const QString& parameter) const;
        bool readCoordinates(const QString& data, int expectedLength, QVector<double>& result) const;
        bool readDouble(const QString& data, double& result) const;

        const QString& m_dot;
        graph::Graph& m_graph;

        DotGraphConfiguration m_graphConfiguration;
        DotNodeConfiguration m_mainNodeConfiguration;
        DotEdgeConfiguration m_mainEdgeConfiguration;
    };
}

#endif // DOTPARSER_H
