#ifndef GRAPHNODE_H
#define GRAPHNODE_H

#include <QSGNode>
#include <QSGFlatColorMaterial>

#include "IGraphNodeConfiguration.h"

namespace divine::gui::graph
{
    class GraphNode : public QSGNode
    {
    public:
        GraphNode();
        GraphNode(const IGraphNodeConfiguration& config);

        const QRectF& getRect() const;
        const QString& getLabel() const;

        /*!
         * \brief Checks whether a point in graph is inside the drawn area.
         * \param local position in the graph
         * \return true if the node shapes contains the given position
         */
        bool containsPosition(const QPointF& position) const;

    private:
        void setBorderColor(const QColor& color);
        void setBackgroundColor(const QColor& color);

        void setRect(const QRectF& rect);

        void updateEllipseGeometry(QSGGeometry& geometry, const QRectF& rect);

        QSGGeometryNode m_ellipseBackgroundNode;
        QSGGeometry m_ellipseBackgroundGeometry;
        QSGFlatColorMaterial m_ellipseBackgroundMaterial;

        QSGGeometryNode m_ellipseBorderNode;
        QSGGeometry m_ellipseBorderGeometry;
        QSGFlatColorMaterial m_ellipseBorderMaterial;

        QRectF m_rect;
        QString m_label;
    };
}

#endif // GRAPHNODE_H
