#ifndef IGRAPHEDGECONFIGURATION_H
#define IGRAPHEDGECONFIGURATION_H

#include "IGraphLabelConfiguration.h"

namespace divine::gui::graph
{
    /*!
     * \brief Represents one edge in the graph.
     *
     * An edge is defined as a set of points defining a cubic B-spline.
     *
     * Every edge can have up to 2 arrows - at the head and at the tail
     * of the edge. If they are included the first and second (or last
     * and next to last) coordinates are taken to create the directional
     * of those arrows.
     *
     * Every edge can also have an label defined in the parent interface.
     */
    class IGraphEdgeConfiguration : virtual public IGraphLabelConfiguration
    {
    public:
        virtual ~IGraphEdgeConfiguration() {}

        /*!
         * \brief Vector of coordinates defining a cubic B-spline.
         *
         * The expected number of coordinates without the arrow coordinates
         * is (n mod 3) == 1. If hasHeadArrow is true there has to be also an
         * additional point at the start of the vector which will define
         * the arrowhead direction. Same for hasTailArrow at the end of
         * the vector.
         *
         * If this condition is met, the first point (which is not the
         * arrowhead point) is taken as the start point. Then three
         * points are cyclicaly taken and used as two controls points
         * and an end point to define the b-spline.
         */
        virtual const QVector<QPointF>& lineCoordinates() const = 0;
        virtual const QColor& lineColor() const = 0;

        virtual bool hasHeadArrow() const = 0;
        virtual bool hasTailArrow() const = 0;
    };
}

#endif // IGRAPHEDGECONFIGURATION_H
