#include "SettingsModel.h"

using namespace divine::gui::models;

SettingsModel::SettingsModel(QObject* parent) :
    QObject(parent),
    m_shortcuts(m_settings)
{
    initializeComponents();
}

ShortcutsModel* SettingsModel::getShortcuts()
{
    return &m_shortcuts;
}

int SettingsModel::componentCount() const
{
    return m_components.size();
}

std::vector<divine::dbg::Component> SettingsModel::debuggedComponents()
{
    return m_includedComponents.keys(true).toVector().toStdVector();
}

QString SettingsModel::getComponentName(int index) const
{
    return m_componentNames[m_components[index]];
}

bool SettingsModel::isComponentIncluded(int index) const
{
    return m_includedComponents[m_components[index]];
}

void SettingsModel::setComponentIncluded(int index, bool value)
{
    dbg::Component component = m_components[index];
    m_includedComponents[component] = value;
    m_settings.setValue(m_componentNames[component], value);
}

void SettingsModel::initializeComponents()
{
    initializeComponent(dbg::Component::Program, "Program", true);
    initializeComponent(dbg::Component::Kernel, "Kernel", false);
    initializeComponent(dbg::Component::DiOS, "DiOS", false);
    initializeComponent(dbg::Component::LibC, "LibC", false);
    initializeComponent(dbg::Component::LibCxx, "LibCxx", false);
    initializeComponent(dbg::Component::LibRst, "LibRst", false);
}

void SettingsModel::initializeComponent(dbg::Component component, QString name, bool defaultValue)
{
    m_components.append(component);
    m_componentNames.insert(component, name);

    bool value = m_settings.value(name, defaultValue).toBool();
    m_includedComponents.insert(component, value);
}

