#ifndef FILESVIEWMODEL_H
#define FILESVIEWMODEL_H

#include <QObject>
#include <QUrl>

#include "Models/SimulatorModel.h"

namespace divine::gui::viewmodels
{
    /*!
     * \brief This simple ViewModels handles storing all the visited files
     * and allowing user to display them. User can also open and display additional files.
     *
     * Files can be accessed from the view by index.
     */
    class FilesViewModel : public QObject
    {
        Q_OBJECT
        Q_PROPERTY(QStringList visitedFiles MEMBER m_visitedFiles NOTIFY visitedFilesChanged)
        Q_PROPERTY(models::SimulatorModel* simulator WRITE setSimulatorModel)
        Q_PROPERTY(int currentIndex MEMBER m_currentIndex NOTIFY currentIndexChanged)
    public:
        FilesViewModel(QObject* parent = nullptr) : QObject(parent)
        {
        }

        void setSimulatorModel(models::SimulatorModel* simulator)
        {
            m_simulator = simulator;
            connect(simulator, &models::SimulatorModel::shownNodeChanged, [&] () {
                sim::DN& shownNode = m_simulator->getShownNode();
                auto shownLinePair = m_simulator->getFileAndLine(shownNode);
                QString file = QString::fromStdString(shownLinePair .first);
                m_currentIndex = addFile(file);
                currentIndexChanged();
            });
        }

    public slots:
        int addFile(const QUrl& url)
        {
            QString file = url.toLocalFile();
            return addFile(file);
        }

        int addFile(const QString& file)
        {
            if (!m_visitedFiles.contains(file))
            {
                m_visitedFiles.append(file);
                visitedFilesChanged();
            }
            return m_visitedFiles.indexOf(file);
        }

        QString getFile(int index) const
        {
            return m_visitedFiles.at(index);
        }

    signals:
        void visitedFilesChanged();
        void currentIndexChanged();

    private:
        QStringList m_visitedFiles;

        models::SimulatorModel* m_simulator;
        int m_currentIndex = -1;
    };
}

#endif // FILESVIEWMODEL_H
