#include "GraphViewModel.h"

#include <QFutureWatcher>

#include "Dot/DotParser.h"

using namespace divine::gui::viewmodels;

GraphViewModel::GraphViewModel(QObject *parent) : QObject(parent)
{
    connect(&m_dotWatcher, &QFutureWatcher<QString>::finished, this, &GraphViewModel::onDotFinished);
}

void GraphViewModel::setSimulator(models::SimulatorModel *simulator)
{
    m_simulator = simulator;
    connect(m_simulator, &models::SimulatorModel::statesChanged, [&](){
       m_isCurrent = false;
    });
}

void GraphViewModel::setGraph(graph::Graph* graph)
{
    m_graph = graph;
    connect(m_graph, &graph::Graph::nodePressed, this, &GraphViewModel::onNodePressed);
}

void GraphViewModel::updateGraph()
{
    if (!m_isCurrent)
    {
        m_graph->clear();

        auto future= m_simulator->getDottedStateGraphAsync();
        m_dotWatcher.setFuture(future);
    }
}

void GraphViewModel::onDotFinished()
{
    auto future = m_dotWatcher.future();
    if (!future.isFinished())
    {
        return;
    }

    dot::DotParser parser(future.result(), *m_graph);
    parser.parse();

    m_isCurrent = true;
    graphUpdated();
}

void GraphViewModel::onNodePressed(const graph::GraphNode &node)
{
    std::string name = "#" + node.getLabel().toStdString();

    auto states = m_simulator->getStates();
    auto iter = std::find_if(std::begin(states), std::end(states), [&name](auto pair) { return pair.second == name; });
    if (iter != std::end(states))
    {
        divine::sim::Snapshot snapshot = iter->first;
        m_simulator->setState(&snapshot);
    }
}
