#ifndef GRAPHVIEWMODEL_H
#define GRAPHVIEWMODEL_H

#include <QFutureWatcher>
#include <QObject>

#include "Models/SimulatorModel.h"
#include "Graph/Graph.h"

namespace divine::gui::viewmodels
{
    /*!
     * \brief ViewModel for the GraphView. Handles displaying and updating the state graph.
     *
     * The state graph is generated on request when the user wants to display it.
     * Once it's generated it will be stored until the states change.
     */
    class GraphViewModel : public QObject
    {
        Q_OBJECT
        Q_PROPERTY(models::SimulatorModel* simulator WRITE setSimulator)
        Q_PROPERTY(graph::Graph* graph WRITE setGraph)
    public:
        explicit GraphViewModel(QObject* parent = nullptr);

        void setSimulator(models::SimulatorModel* simulator);
        void setGraph(graph::Graph* graph);
    public slots:
        void updateGraph();

    signals:
        void graphUpdated();

    private:
        void onDotFinished();
        void onNodePressed(const graph::GraphNode& node);

        models::SimulatorModel* m_simulator = nullptr;
        graph::Graph* m_graph = nullptr;

        bool m_isCurrent = false;
        QFutureWatcher<QString> m_dotWatcher;

    };
}

#endif // GRAPHVIEWMODEL_H
