#ifndef MAINVIEWMODEL_H
#define MAINVIEWMODEL_H

#include <QCoreApplication>
#include <QObject>

#include <cstdlib>
#include <thread>

#include "Models/SimulatorModel.h"

namespace divine::gui::viewmodels
{
    /*!
     * \brief The ViewModel for the main window itself.
     *
     * This very simple ViewModel handles the initialization of the program
     * and provides texts for the About dialog.
     */
    class MainViewModel : public QObject
    {
        Q_OBJECT
    public:
        MainViewModel(QObject* parent = nullptr) : QObject(parent)
        {
        }

    public slots:
        bool initialize(models::SimulatorModel* simulator)
        {
            QStringList args = QCoreApplication::arguments();
            if (args.size() > 1)
            {
                simulator->loadFile(args.at(1));
                return true;
            }
            return false;
        }

        void openNewFile(const QString& path)
        {
            QString pathToProgram = QCoreApplication::arguments().at(0);
            QString command = pathToProgram + " " + path;
            std::thread([command](){
                std::system(command.toLatin1().data());
            }).detach();
        }

        QString getAboutText()
        {
            return tr("DIVINE is free software, distributed under the ISC licence (2-clause BSD).\n"
                      "Graphical interface is built using the Qt framework. When linked with Qt LGPLv3 applies.");
        }

        QString getAdditionalText() {
            return tr("Please refer to doc/AUTHORS and doc/COPYING included in the distribution for co­pyright and licensing details.\n\n"
                      "The program is provided AS IS with NO WARRANTY OF ANY KIND,\n"
                      "INCLUDING THE WARRANTY OF DESIGN, MERCHANTABILITY\n"
                      "AND FITNESS FOR A PARTICULAR PURPOSE.");
        }
    };
}

#endif // MAINVIEWMODEL_H
