#include "StackTraceViewModel.h"

using namespace divine::gui::viewmodels;

StackTraceViewModel::StackTraceViewModel(QObject *parent) : QAbstractItemModel(parent)
{
}

divine::gui::models::SimulatorModel* StackTraceViewModel::simulator() const
{
    return m_simulator;
}

void StackTraceViewModel::setSimulator(models::SimulatorModel *simulator)
{
    m_simulator = simulator;
    connect(simulator, &models::SimulatorModel::stackTraceChanged, this, &StackTraceViewModel::onStackTraceChanged);
}

QHash<int, QByteArray> StackTraceViewModel::roleNames() const
{
    return QHash<int, QByteArray>
    {
        { FunctionRole, "functionName" },
        { LineRole, "line" },
        { FileRole, "file" }
    };
}

QModelIndex StackTraceViewModel::index(int row, int column, const QModelIndex &parent) const
{
    return row < m_stackTrace.count() ? createIndex(row, column, nullptr) : QModelIndex();
}

QModelIndex StackTraceViewModel::parent(const QModelIndex &child) const
{
    return QModelIndex();
}

bool StackTraceViewModel::hasChildren(const QModelIndex &parent) const
{
    return false;
}

int StackTraceViewModel::rowCount(const QModelIndex &parent) const
{
    if (!parent.isValid())
    {
        return m_stackTrace.count();
    }
    return 0;
}

int StackTraceViewModel::columnCount(const QModelIndex &parent) const
{
    return 1;
}

QVariant StackTraceViewModel::data(const QModelIndex &index, int role) const
{
    if (!index.isValid())
    {
        return QVariant();
    }

    switch(role){
        case (FunctionRole):
            return getFunctionName(index.row());
        case (LineRole):
            return getLineNumber(index.row());
        case (FileRole):
            return getFileName(index.row());
    }
    return QVariant();
}

int StackTraceViewModel::stackTraceRowCount() const
{
    return m_stackTrace.count();
}

void StackTraceViewModel::goToFile(int index)
{
    m_simulator->setShownNode(m_stackTrace[index]);
}

void StackTraceViewModel::onStackTraceChanged()
{
    beginResetModel();
    m_stackTrace.clear();
    m_stackTrace = m_simulator->getStackTrace();
    endResetModel();
    stackTraceChanged();
}

QString StackTraceViewModel::getFunctionName(int index) const
{
    if (index >= 0 && index < m_stackTrace.count())
    {
        return QString::fromStdString(m_simulator->getFunctionName(m_stackTrace[index]));
    }
    return QString();
}

QString StackTraceViewModel::getLineNumber(int index) const
{
    if (index >= 0 && index < m_stackTrace.count())
    {
        auto pair = m_simulator->getFileAndLine(m_stackTrace[index]);
        return QString::number(pair.second);
    }
    return QString();
}

QString StackTraceViewModel::getFileName(int index) const
{
    if (index >= 0 && index < m_stackTrace.count())
    {
        auto pair = m_simulator->getFileAndLine(m_stackTrace[index]);
        return QString::fromStdString(pair.first);
    }
    return QString();
}
