#ifndef STACKTRACEVIEWMODEL_H
#define STACKTRACEVIEWMODEL_H

#include <QAbstractItemModel>
#include <QList>

#include "Models/SimulatorModel.h"

namespace divine::gui::viewmodels
{
    /*!
     * \brief ViewModel for the stack trace view.
     *
     * The view contains three columns specified by the enum Role.
     * QAbstractItemModel is implemented to allow access to those columns.
     */
    class StackTraceViewModel : public QAbstractItemModel
    {
        Q_OBJECT
        Q_PROPERTY(models::SimulatorModel* simulator READ simulator WRITE setSimulator)
    public:
        StackTraceViewModel(QObject* parent = nullptr);

        models::SimulatorModel* simulator() const;
        void setSimulator(models::SimulatorModel* simulator);

        // Qt overrides left undocumented - Qt docs are enough
        QHash<int, QByteArray> roleNames() const override;
        QModelIndex index(int row, int column, const QModelIndex &parent = QModelIndex()) const override;
        QModelIndex parent(const QModelIndex& child) const override;
        bool hasChildren(const QModelIndex& parent) const;
        int rowCount(const QModelIndex& parent) const override;
        int columnCount(const QModelIndex &parent) const override;
        QVariant data(const QModelIndex &index, int role) const override;
        int stackTraceRowCount() const;
    public slots:
        /*!
         * \brief Sets the node represented by the given index as the current shown node.
         * \param Index to show.
         */
        void goToFile(int index);

    signals:
        void stackTraceChanged();

    private:
        void onStackTraceChanged();

        QString getFunctionName(int index) const;
        QString getLineNumber(int index) const;
        QString getFileName(int index) const;

        enum Role {
            FunctionRole = Qt::DisplayRole + 1,
            LineRole,
            FileRole
        };

        models::SimulatorModel* m_simulator;
        QList<sim::DN> m_stackTrace;
    };
}

#endif // STACKTRACEVIEWMODEL_H
