#include "StateViewModel.h"

using namespace divine::gui::viewmodels;

StateViewModel::StateViewModel(QObject *parent) : QObject(parent)
{

}

divine::gui::models::SimulatorModel* StateViewModel::simulator() const
{
    return m_simulator;
}

void StateViewModel::setSimulator(models::SimulatorModel* simulator)
{
    m_simulator = simulator;
    connect(simulator, &models::SimulatorModel::statesChanged, this, &StateViewModel::onStatesChanged);
}

int StateViewModel::stateCount() const
{
    return m_states.size();
}

QString StateViewModel::getStateName(int index) const
{
    return m_states.keys().value(index, QString());
}

void StateViewModel::goToState(int index)
{
    if (index >= 0 && index < m_states.size())
    {
        m_simulator->setState(&m_states.values()[index]);
    }
}

void StateViewModel::captureState()
{
    m_simulator->captureState();
}

void StateViewModel::goToLastState()
{
    m_simulator->setLastState();
}

void StateViewModel::onStatesChanged()
{
    auto states = m_simulator->getStates();
    // reverse keys and values, names should be unique so it's fine
    m_states.clear();
    for (auto it = states.begin(); it != states.end(); ++it)
    {
        m_states.insert(QString::fromStdString(it->second), it->first);
    }
    stateCountChanged();
}
