#ifndef STATEVIEWMODEL_H
#define STATEVIEWMODEL_H

#include <QObject>
#include <QMap>
#include <QString>

#include "Models/SimulatorModel.h"

namespace divine::gui::viewmodels
{
    /*!
     * \brief Simple ViewModel for the state view.
     *
     * States in this ViewModel are represented by indexes.
     */
    class StateViewModel : public QObject
    {
        Q_OBJECT
        Q_PROPERTY(models::SimulatorModel* simulator READ simulator WRITE setSimulator)
        Q_PROPERTY(int stateCount READ stateCount NOTIFY stateCountChanged)
    public:
        explicit StateViewModel(QObject *parent = nullptr);

        models::SimulatorModel* simulator() const;
        void setSimulator(models::SimulatorModel* simulator);

        int stateCount() const;

    signals:
        void stateCountChanged() const;

    public slots:
        QString getStateName(int index) const;

        void goToState(int index);

        void captureState();
        void goToLastState();

    private:
        void onStatesChanged();

        models::SimulatorModel* m_simulator;
        QMap<QString, sim::Snapshot> m_states;
    };
}
#endif // STATEVIEWMODEL_H
