import QtQuick 2.10
import QtQuick.Controls 2.3
import QtQuick.Layouts 1.3

import divine.gui 1.0

Item {
    id: rootItem
    property SettingsModel settings
    property bool showBitcode: false

    RowLayout {
        spacing: 5
        width: parent.width

        ComboBox {
            id: modeComboBox

            Layout.leftMargin: 32
            Layout.minimumWidth: 160
            textRole: 'key'

            model: ListModel {
                id: stepSelectionModel

                ListElement { key: qsTr("Step line"); value: SimulatorModel.Line }
                ListElement { key: qsTr("Step atomic"); value: SimulatorModel.Atomic }
                ListElement { key: qsTr("Step instruction"); value: SimulatorModel.Instruction }
            }

            MouseArea {
                anchors.fill: parent
                anchors.rightMargin: parent.indicator.width

                onClicked: {
                    var element = stepSelectionModel.get(parent.currentIndex);
                    viewModel.simulator.step(element.value, SimulatorModel.In);
                }
            }
        }

        ActionGroup {
            id: actionGroup
            exclusive: false
        }
        Repeater {
            id: actions
            model: 9

            property var shortcuts: [ settings.shortcuts.stepLineOver, settings.shortcuts.stepLineIn,  settings.shortcuts.stepLineOut,
                settings.shortcuts.stepInstructionOver, settings.shortcuts.stepInstructionIn,  settings.shortcuts.stepInstructionOut,
                settings.shortcuts.stepAtomicOver, settings.shortcuts.stepAtomicIn,  settings.shortcuts.stepAtomicOut ]
            property var texts: [ qsTr('Over'), qsTr('In'), qsTr('Out') ]

            delegate: Item {
                Action {
                    ActionGroup.group: actionGroup

                    shortcut: actions.shortcuts[index]
                    text: actions.texts[index % actions.texts.length]

                    onTriggered: viewModel.simulator.step(index / 3, index % 3);
                }
            }
        }

        // cannot use Repeater for these Buttons due to QTBUG-65889
        Button {
            Layout.fillWidth: false
            Layout.leftMargin: 10

            action: actionGroup.actions[modeComboBox.currentIndex * 3]

            ShortcutTooltip {}
        }
        Button {
            Layout.fillWidth: false
            Layout.leftMargin: 10

            action: actionGroup.actions[modeComboBox.currentIndex * 3 + 1]

            ShortcutTooltip {}
        }
        Button {
            Layout.fillWidth: false
            Layout.leftMargin: 10

            action: actionGroup.actions[modeComboBox.currentIndex * 3 + 2]

            ShortcutTooltip {}
        }

        Button {
            Layout.fillWidth: false
            Layout.leftMargin: 10

            action: Action {
                shortcut: settingsModel.shortcuts.callContinue
                text: qsTr('Continue')

                onTriggered: viewModel.callContinue()
            }

            ShortcutTooltip {}
        }

        // separator
        Item {
            Layout.fillWidth: true
        }

        Button {
            Layout.fillWidth: false
            Layout.alignment: Qt.AlignRight

            action: Action {
                text: rootItem.showBitcode ? qsTr("Source") : qsTr("Bitcode")
                shortcut: settingsModel.shortcuts.switchBitcode
                onTriggered: rootItem.showBitcode = !rootItem.showBitcode
            }

            ShortcutTooltip {}
        }
    }    

    ExecutionControlViewModel {
       id: viewModel
    }

    property alias viewModel: viewModel
}
