import QtQuick 2.10
import QtQuick.Controls 2.3
import QtQuick.Layouts 1.3
import QtQuick.Dialogs 1.2
import QtQuick.Controls 1.4 as Controls1

import divine.gui 1.0

ColumnLayout
{
    property SettingsModel settings
    property alias viewModel: viewModel
    signal fileRequested(string file)

    spacing: 0

    Controls1.TableView {
        id: filesTable

        Layout.fillHeight: true
        Layout.fillWidth: true
        frameVisible: false

        model: viewModel.visitedFiles

        onClicked: fileRequested(viewModel.getFile(row))

        Controls1.TableViewColumn {
            id: functionColumn
            title: qsTr('File')
            role: 'modelData'
        }
    }

    Button {
       Layout.preferredHeight: 35
       Layout.alignment: Qt.AlignCenter

       action: Action {
            text: qsTr('Open')
            icon.name: 'list-add'
            shortcut: settings.shortcuts.addFile
            onTriggered: openFileDialog.open()
        }

        ShortcutTooltip {}
    }

    FileDialog {
        id: openFileDialog

        selectExisting: true
        selectFolder: false
        selectMultiple: false

        nameFilters: [qsTr("C/C++ files (*.cpp *.cc *.C *.CPP *.c++ *.cp *.cxx)")]
        folder: shortcuts.home

        onAccepted: {
            var index = viewModel.addFile(fileUrl);
            fileRequested(viewModel.getFile(index));
            filesTable.selection.clear();
            filesTable.selection.select(index);
        }
    }

    FilesViewModel {
        id: viewModel

        onCurrentIndexChanged: {
            filesTable.selection.clear();
            filesTable.selection.select(viewModel.currentIndex);
        }
    }
}
