import QtQuick 2.10
import QtQuick.Controls 2.3
import QtQuick.Layouts 1.3

import divine.gui 1.0

GridLayout {
    id: rootGrid
    rows: 5
    columns: 2

    property SimulatorModel simulatorModel

    readonly property double minimumScale: 0.1
    readonly property int buttonSize: 50

    onVisibleChanged: {
        if (visible){
            viewModel.updateGraph();
        }
    }

    Button {
        Layout.preferredHeight: buttonSize
        Layout.preferredWidth: buttonSize

        action: Action {
            icon.name: 'zoom-in'
            onTriggered: {
                scaler.xScale *= 1.1;
                scaler.yScale *= 1.1;
            }
        }
        display: AbstractButton.IconOnly
    }

    Item {
        id: graphViewClipper

        Layout.rowSpan: 5
        Layout.fillHeight: true
        Layout.fillWidth: true
        Layout.margins: 5

        clip: true        

        MouseArea
        {
            anchors.fill: parent
            hoverEnabled: true
            propagateComposedEvents: true
            drag.target: graph
            drag.filterChildren: true

            onWheel: {
                var oldX = scaler.origin.x;
                var oldY = scaler.origin.y;
                var oldZoom = scaler.xScale;

                var mappedMouse = graphViewClipper.mapToItem(graph, mouseX, mouseY);
                scaler.origin.x = mappedMouse.x;
                scaler.origin.y = mappedMouse.y;

                if ((scaler.xScale + wheel.angleDelta.y / 1800) > minimumScale)
                {
                    scaler.xScale += wheel.angleDelta.y / 1800;
                    scaler.yScale += wheel.angleDelta.y / 1800;
                }
                else
                {
                    scaler.xScale = minimumScale;
                    scaler.yScale = minimumScale;
                }
                graph.x += (oldX-scaler.origin.x)*(1-oldZoom);
                graph.y += (oldY-scaler.origin.y)*(1-oldZoom);
            }

            Graph {
                id: graph
                antialiasing: true

                transform: Scale {
                    id: scaler
                }
            }
        }
    }

    Button {
        Layout.preferredHeight: buttonSize
        Layout.preferredWidth: buttonSize

        action: Action {
            icon.name: 'zoom-out'
            onTriggered: {
                if ((scaler.xScale * 0.9) > minimumScale)
                {
                    scaler.xScale *= 0.9;
                    scaler.yScale *= 0.9;
                }
                else
                {
                    scaler.xScale = minimumScale;
                    scaler.yScale = minimumScale;
                }
            }
        }
        display: AbstractButton.IconOnly
    }

    Button {
        Layout.preferredHeight: buttonSize
        Layout.preferredWidth: buttonSize

        action: Action {
            id: bestFitAction
            icon.name: 'zoom-fit-best'
            onTriggered: {
                // we use the same scale for both axes, so use the minimal one
                var scale = Math.min(graphViewClipper.width / graph.width, graphViewClipper.height / graph.height);
                scale = Math.max(scale, minimumScale);
                // center and scale
                scaler.origin.x = graphViewClipper.width / 2;
                scaler.origin.y = graphViewClipper.height / 2;
                scaler.xScale = scale;
                scaler.yScale = scale;
                // move to 0,0 in the scaled space
                graph.x = -scaler.origin.x + scaler.origin.x * scaler.xScale;
                graph.y = -scaler.origin.y + scaler.origin.y * scaler.yScale;
            }
        }
        display: AbstractButton.IconOnly
    }

    Button {
        Layout.preferredHeight: buttonSize
        Layout.preferredWidth: buttonSize
        anchors.bottom: rootGrid.bottom

        action: Action {
            icon.name: 'window-cse'
            onTriggered: rootGrid.visible = false
        }
        display: AbstractButton.IconOnly
    }

    GraphViewModel {
        id: viewModel
        simulator: simulatorModel
        graph: graph

        onGraphUpdated: {
            graph.update();
            bestFitAction.trigger();
        }
    }
}
