﻿import QtQuick 2.10
import QtQuick.Controls 2.3
import QtQuick.Layouts 1.3
import QtQuick.Window 2.3

import divine.gui 1.0

Window {
    modality: Qt.ApplicationModal
    minimumWidth: 600
    minimumHeight: 500

    property SettingsModel settingsModel

    ColumnLayout {
        anchors.fill: parent

        TabBar {
            id: bar

            Layout.fillWidth: true

            Repeater {
                model: [qsTr("General"), qsTr("Shortcuts")]

                TabButton {
                    text: modelData
                    width: Math.max(100, bar.width / parent.count)
                }
            }
        }

        StackLayout {
            currentIndex: bar.currentIndex

            Layout.fillWidth: true
            Layout.fillHeight: true
            clip: true

            ColumnLayout {
                id: generalTab

                ListView {
                    model: settingsModel.componentCount

                    Layout.fillWidth: true
                    Layout.fillHeight: true

                    header: Text {
                        text: qsTr('Debugged components')
                        font.bold: true
                    }

                    delegate: Switch {
                        text: settingsModel.getComponentName(index)
                        checked: settingsModel.isComponentIncluded(index)
                        onCheckedChanged: settingsModel.setComponentIncluded(index, checked)
                    }
                }
            }

            ListView {
                id: shortcutsTab
                Layout.fillWidth: true
                Layout.fillHeight: true

                model: settingsModel.shortcuts

                section.property: 'category'
                section.delegate: headerComponent

                delegate: shortcutRowComponent
            }
        }
    }

    Component {
        id: headerComponent

        Text {
            text: section
            font.bold: true
        }
    }

    Component {
        id: shortcutRowComponent

        MouseArea {
            id: shortcutRowMouseArea
            width: parent.width
            height: 20
            hoverEnabled: true

            Row {
                spacing: 10
                width: parent.width
                height: parent.height

                property bool isRecording: false
                property var combination: []

                Text {
                    text: name
                    width: parent.width / 2
                    verticalAlignment: Text.AlignVCenter
                }
                Text {
                    id: shortcutValueLabel
                    text: value
                    verticalAlignment: Text.AlignVCenter
                }
                Button {
                    text: parent.isRecording ? qsTr('Finish') : qsTr('Record')
                    height: 25
                    visible: shortcutRowMouseArea.containsMouse || parent.isRecording

                    onClicked: {
                        if (parent.isRecording) {
                            parent.isRecording = false;
                            parent.combination = [];
                        }
                        else
                        {
                            parent.isRecording = true;
                        }
                    }

                    Keys.enabled: parent.isRecording
                    Keys.onPressed: {
                        var key = event.key;
                        if (key === Qt.Key_Control || key === Qt.Key_Shift
                            || key === Qt.Key_Meta || key === Qt.Key_Alt || key === Qt.Key_unknown)
                        {
                            return;
                        }

                        if (parent.isRecording)
                        {
                            parent.combination.push(key | event.modifiers);
                            settingsModel.shortcuts.setShortcutValue(index, parent.combination);

                            if (parent.combination.length === 4)
                            {
                                parent.isRecording = false;
                                parent.combination = [];
                            }
                            event.accepted = true;
                        }
                    }
                    Keys.onReleased: {
                        if (parent.isRecording)
                        {
                            event.accepted = true;
                        }
                    }
                }
                Button {
                    text: qsTr('Reset')
                    height: 25
                    visible: shortcutRowMouseArea.containsMouse || parent.isRecording

                    onClicked: {
                        if (parent.isRecording) {
                            parent.isRecording = false;
                            parent.combination = [];
                        }

                        settingsModel.shortcuts.resetShortcutValue(index);
                    }
                }
            }
        }
    }
}
