#include <QGuiApplication>
#include <QQmlApplicationEngine>
#include <QQuickStyle>
#include <QSurfaceFormat>

#include "ViewModels/MainViewModel.h"
#include "ViewModels/CodeViewModel.h"
#include "ViewModels/ExecutionControlViewModel.h"
#include "ViewModels/StackTraceViewModel.h"
#include "ViewModels/DataViewModel.h"
#include "ViewModels/StateViewModel.h"
#include "ViewModels/FilesViewModel.h"
#include "ViewModels/GraphViewModel.h"

#include "Models/SettingsModel.h"

#include "Graph/Graph.h"

using namespace divine::gui;

int main(int argc, char *argv[])
{
    QGuiApplication::setAttribute(Qt::AA_EnableHighDpiScaling);
    QGuiApplication app(argc, argv);

    QIcon icon(QStringLiteral(":/divine.png"));
    app.setWindowIcon(icon);

    QCoreApplication::setOrganizationName("ParaDiSe FI MUNI");
    QCoreApplication::setOrganizationDomain("paradise.fi.muni.cz");
    QCoreApplication::setApplicationName("DIVINE");

    QQuickStyle::setFallbackStyle("Material");

    qmlRegisterType<models::SimulatorModel>("divine.gui", 1, 0, "SimulatorModel");
    qmlRegisterType<models::SettingsModel>("divine.gui", 1, 0, "SettingsModel");

    qmlRegisterType<viewmodels::MainViewModel>("divine.gui", 1, 0, "MainViewModel");
    qmlRegisterType<viewmodels::CodeViewModel>("divine.gui", 1, 0, "CodeViewModel");
    qmlRegisterType<viewmodels::ExecutionControlViewModel>("divine.gui", 1, 0, "ExecutionControlViewModel");
    qmlRegisterType<viewmodels::StackTraceViewModel>("divine.gui", 1, 0, "StackTraceViewModel");
    qmlRegisterType<viewmodels::DataViewModel>("divine.gui", 1, 0, "DataViewModel");
    qmlRegisterType<viewmodels::StateViewModel>("divine.gui", 1, 0, "StateViewModel");
    qmlRegisterType<viewmodels::FilesViewModel>("divine.gui", 1, 0, "FilesViewModel");
    qmlRegisterType<viewmodels::GraphViewModel>("divine.gui", 1, 0, "GraphViewModel");

    qmlRegisterType<graph::Graph>("divine.gui", 1, 0, "Graph");

    QSurfaceFormat fmt;
    fmt.setVersion(2, 0);
    fmt.setSamples(4);
    fmt.setProfile(QSurfaceFormat::CoreProfile);
    QSurfaceFormat::setDefaultFormat(fmt);

    QQmlApplicationEngine engine(QUrl(QStringLiteral("qrc:/Views/MainView.qml")));
    return app.exec();
}
